// NF logging system implementation
#include "pch.h"

#include "log.h"

namespace nf::util {
    // Log is only present in debug builds
#ifdef _DEBUG

    // Mutex to synchronize access to stdout
    static std::mutex s_logMutex;
    // Needed to enable Win32 virtual terminal processing
    static bool s_isLogInit = false;
    // Mapping of type strings
    static std::map<LogType, const char*> s_logTypesMap = {
        {LogType::Log, "\x1b[93mLog\x1b[0m"},
        {LogType::Error, "\x1b[91mError\x1b[0m"}
    };

    static void initLog() {
        HANDLE stdoutHandle = GetStdHandle(STD_OUTPUT_HANDLE);
        DWORD consoleMode = 0;
        GetConsoleMode(stdoutHandle, &consoleMode);
        consoleMode |= ENABLE_VIRTUAL_TERMINAL_PROCESSING;
        SetConsoleMode(stdoutHandle, consoleMode);
        s_isLogInit = true;
    }

    void log(const char* msg, LogType type) {
        std::lock_guard logLock(s_logMutex);

        if (!s_isLogInit)
            initLog();

        std::cout << std::format("NF {}: {}\n", s_logTypesMap[type], msg);
    }

    void log(const std::string& msg, LogType type) {
        log(msg.c_str(), type);
    }

#endif

    void error(const char* msg, const char* file, unsigned int line) {
        std::string filename = file;
        filename = filename.substr(filename.rfind("\\src\\") + 5);

#ifdef _DEBUG
        log(std::format("({}, {}) {}", filename, line, msg), LogType::Error);
        MessageBeep(0);
        __debugbreak();
#else
        std::string fmtMsg = std::format("{}\n\n({} at line {})", msg, filename, line);
        MessageBox(nullptr, fmtMsg.c_str(), "Engine Error", MB_OK | MB_ICONERROR);
        std::exit(1);
#endif

    }
}
